
/*
 * Copyright (C) 2024 Web Server LLC
 */


#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_string.h>

#include <ngx_wasm.h>
#include <ngx_wasm_event.h>
#include <wasi/ngx_wasi_event.h>


#define NGX_WASM_HOST_POLL_TAG 0x504F4C4C /* POLL */


/* TODO: serialization format and what fields are needed to pass events */
typedef struct {
    uint32_t             fd;
    uint32_t             flags;
    uint32_t             ptr;
} my_serial_ev_t;


static int32_t ngx_wasm_host_open_events(ngx_wasm_handle_t *self,
    ngx_wasm_handle_t *parent);
static int32_t ngx_wasm_event_property_get(ngx_wasm_handle_t *self,
    uint32_t propid, uint8_t *buf, uint32_t len);


ngx_wasm_handle_ops_t ngx_wasm_event_ops[] = {
    {   .type = ngx_string("ngx::core::events"),
        .tag = NGX_WASM_HOST_POLL_TAG,
        .open = ngx_wasm_host_open_events,
        .read = ngx_wasm_host_io_stub,
        .write = ngx_wasm_host_io_stub,
        .get = ngx_wasm_event_property_get,
        .set = ngx_wasm_host_property_stub,
        .close = ngx_wasm_host_close_stub,
    },

    ngx_wasm_host_null_ops
};


static int32_t
ngx_wasm_host_open_events(ngx_wasm_handle_t *self, ngx_wasm_handle_t *parent)
{
    /* we operate on host directly, no specific fd state */
    /* maybe: may keep some stats here */
    return NGX_OK;
}


static int32_t
ngx_wasm_event_property_get(ngx_wasm_handle_t *self, uint32_t propid,
    uint8_t *buf, uint32_t len)
{
    ngx_queue_t       *q;
    my_serial_ev_t    *sev;
    ngx_wasm_event_t  *we;

    switch (propid) {

    case NGX_WASI_EVENT_OPS_GET:
        ngx_log_debug1(NGX_LOG_DEBUG_HTTP, self->log, 0,
                       "%V get event", &self->host->name);

        if (ngx_queue_empty(&self->host->events)) {
            return 0;
        }

        if (len < sizeof(my_serial_ev_t)) {
            ngx_log_error(NGX_LOG_ERR, self->log, 0, "%V buffer is too small",
                          &self->host->name);
            return NGX_ERROR;
        }

        q = ngx_queue_head(&self->host->events);

        we = ngx_queue_data(q, ngx_wasm_event_t, queue);

        sev = (my_serial_ev_t *) buf;

        sev->fd = we->fd;
        sev->flags = we->flags;
        sev->ptr = we->ptr;

        /* event is processed */
        ngx_queue_remove(q);
        ngx_queue_insert_head(&self->host->free_events, q);

        return sizeof(my_serial_ev_t);

    default:
        ngx_log_error(NGX_LOG_ERR, self->log, 0,
                      "%V unknown request property %D",
                      &self->host->name, propid);

        return NGX_ERROR;
    }

    /* unreachable */
    return NGX_ERROR;
}
