
/*
 * Copyright (C) 2024 Web Server LLC
 */


#ifndef _NGX_WASM_HANDLE_H_INCLUDED_
#define _NGX_WASM_HANDLE_H_INCLUDED_


#include <ngx_config.h>
#include <ngx_core.h>


#define NGX_WASM_API_CONTEXT 0xFFCA


/* internal object handle with state identified by descriptor */
struct ngx_wasm_handle_s {
    ngx_rbtree_node_t         node;
    ngx_queue_t               queue; /* in free list */
    uint32_t                  fd;    /* assigned descriptor number */
    ngx_wasm_host_t          *host;  /* host env of this handle */
    ngx_wasm_handle_ops_t    *ops;   /* type-specific operations */
    ngx_uint_t                tag;   /* set by host object or by open */
    ngx_log_t                *log;
    uint32_t                  rev_handler;
    uint32_t                  wev_handler;
    void                     *data;  /* internal state, created by open() */
};


/* actual operations on host's objects */
struct ngx_wasm_handle_ops_s {
    ngx_str_t    type;  /* object name */
    ngx_uint_t   tag;
    int32_t    (*open)(ngx_wasm_handle_t *self, ngx_wasm_handle_t *ctx);
    int32_t    (*close)(ngx_wasm_handle_t *self);
    int32_t    (*read)(ngx_wasm_handle_t *self, uint8_t *buf, uint32_t len);
    int32_t    (*write)(ngx_wasm_handle_t *self, uint8_t *buf, uint32_t len);
    int32_t    (*get)(ngx_wasm_handle_t *self, uint32_t propid, uint8_t *buf,
                      uint32_t len);
    int32_t    (*set)(ngx_wasm_handle_t *self, uint32_t propid, uint8_t *buf,
                      uint32_t len);
};


ngx_wasm_handles_t *ngx_wasm_handles_create(ngx_pool_t *pool);
ngx_wasm_handle_t *ngx_wasm_find_handle(ngx_wasm_handles_t *handles,
    int32_t fd);
ngx_wasm_handle_t *ngx_wasm_create_handle(ngx_wasm_handles_t *handles);
void ngx_wasm_delete_handle(ngx_wasm_handles_t *handles,
    ngx_wasm_handle_t *handle);


#endif /* _NGX_WASM_HANDLE_H_INCLUDED_ */
