
/*
 * Copyright (C) 2024 Web Server LLC
 */


#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_wasm_log.h>


static int32_t ngx_wasm_host_open_log(ngx_wasm_handle_t *self,
    ngx_wasm_handle_t *parent);
static int32_t ngx_wasm_host_write_log(ngx_wasm_handle_t *self, uint8_t *buf,
    uint32_t len);

ngx_wasm_handle_ops_t ngx_wasm_log_ops[] = {
    {   .type = ngx_string("ngx::core::log"),
        .tag = NGX_WASM_HOST_LOG_TAG,
        .open = ngx_wasm_host_open_log,
        .read = ngx_wasm_host_io_stub,
        .write = ngx_wasm_host_write_log,
        .get = ngx_wasm_host_property_stub,
        .set = ngx_wasm_host_property_stub,
        .close = ngx_wasm_host_close_stub,
    },

    ngx_wasm_host_null_ops
};


static int32_t
ngx_wasm_host_open_log(ngx_wasm_handle_t *self, ngx_wasm_handle_t *parent)
{
    self->data = self->host->log;

    return NGX_OK;
}


static int32_t
ngx_wasm_host_write_log(ngx_wasm_handle_t *self, uint8_t *buf, uint32_t len)
{
    ngx_log_t *log = self->data;

    ngx_str_t           msg;
    ngx_uint_t          level;
    ngx_log_handler_pt  handler;

    if (len <= 1) {
        return 0;
    }

    level = *buf++;

    if (level > NGX_LOG_DEBUG) {
        level = NGX_LOG_DEBUG;
    }

    msg.data = (u_char *) buf;
    msg.len = len - 1;


    /* avoid details, if any */
    handler = log->handler;
    log->handler = NULL;

    if (level == NGX_LOG_DEBUG) {
        ngx_log_debug2(NGX_LOG_DEBUG_HTTP, log, 0,
                       "%V guest: %V", &self->host->name, &msg);

    } else {
        ngx_log_error(level, log, 0, "%V guest: %V", &self->host->name, &msg);
    }

    log->handler = handler;

    return len;
}
